@@ -130,7 +130,8 @@ def _setup_env(deployment_tag=None, environment=None, override_servers=None):
     env.static_root = os.path.join(env.root, 'static_media')
     env.services = os.path.join(env.home, 'services')
     env.nginx_conf = os.path.join(env.services, 'nginx', '%s.conf' % env.environment)
-    env.local_settings_py = os.path.join(env.project_root, 'local_settings.py')
+    env.setdefault('local_settings_py_relative_path', 'local_settings.py')
+    env.local_settings_py = os.path.join(env.project_root, env.local_settings_py_relative_path)
     env.database_name = '%s_%s' % (env.project, env.environment)
     env.staticfiles_s3_bucket = '-'.join([env.deployment_tag, env.environment, 'static', 'files'])
     env.site_domains = env.site_domains_map[env.environment]
@@ -1420,7 +1421,7 @@ def executel(cmd, *args, **kwargs):
         name = cmd.name.upper()
     else:
         name = str(cmd).upper()
-    arguments = [str(v) for v in args] + ['%s=%s' % (k, v) for k, v in kwargs.items]
+    arguments = [str(v) for v in args] + ['%s=%s' % (k, v) for k, v in kwargs.items()]
     logger.info('\n\n **** %s (%s) ****\n\n' % (name, ', '.join(arguments)))
     execute(cmd, *args, **kwargs)
 
@@ -2039,7 +2040,7 @@ def _refresh_instances(autoscaling_group):
                         # 30 didn't seem to be long enough.
         # If this is the first instance we're adding, make sure at least one new
         # one comes up before continuing.
-        for i in range(max_inst_create_time/check_period):
+        for i in range(int(max_inst_create_time/check_period)):
             time.sleep(check_period)
             autoscaling_group = _get_autoscaling_group()  # Refresh instances list.
             # Wait for a new instance, if any, to be fully brought up.
